
#include <lubishev/Common.h>
#include <lubishev/DBDRecords.h>
#include <lubishev/Exception.h>

#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QTextStream>
#include <QUuid>

#include <iostream>

namespace lubishev {

QString DBDRecords::createEvent(time_t start, time_t end) {
    QString id = createID(start);
    ensureDirs(id);
    QString fileName(absPath(id));
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not create event file '%s'", qPrintable(fileName)));
    QTextStream out(&file);
    out << start << "\n" << end << "\n";
    return id;
}

QStringList DBDRecords::eventTags(const QString &id) {
    QString fileName(absPath(id));
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not open event file '%s'", qPrintable(fileName)));
    QTextStream in(&file);
    // Skip timestamps.
    in.readLine();
    in.readLine();
    QString ln;
    QStringList list;
    while (true)
    {
        ln = in.readLine();
        if (!ln.isNull())
            list.push_back(ln);
        else
            break;
    }
    return list;
}

QStringList DBDRecords::eventsWithin(time_t start, time_t end) {
    QStringList list = dateFiles(start) + dateFiles(end);
    list.removeDuplicates();
    return list;
}

void DBDRecords::eventTimes(const QString &id, time_t *start, time_t *end) {
    QString fileName(absPath(id));
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not open event file '%s'", qPrintable(fileName)));
    QTextStream in(&file);
    QString a = in.readLine();
    if (start)
        *start = a.toULong();
    a = in.readLine();
    if (end)
        *end = a.toULong();
}

void DBDRecords::setDir(const QString &dir) {
    mDir = dir;
}

void DBDRecords::setEventTags(const QString &id, const QStringList &list) {
    QString fileName(absPath(id));
    QFile file(fileName);
    // Read timestamps.
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not open event file '%s'", qPrintable(fileName)));
    QString start;
    QString end;
    {
        QTextStream in(&file);
        start = in.readLine();
        end   = in.readLine();
    }
    file.close();
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not open event file '%s'", qPrintable(fileName)));
    QTextStream out(&file);
    // Write data.
    out << start << "\n" << end << "\n";
    for (QStringList::const_iterator it = list.begin(); it != list.end(); ++it)
        out << *it << "\n";
}

void DBDRecords::setEventTimes(const QString &id, time_t start, time_t end) {
    QString fileName(absPath(id));
    QFile file(fileName);
    // Read tags.
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not open event file '%s'", qPrintable(fileName)));
    QStringList tags;
    {
        QTextStream in(&file);
        // Skip timestamps.
        in.readLine();
        in.readLine();
        QString ln;
        while (true)
        {
            ln = in.readLine();
            if (!ln.isNull())
                tags.push_back(ln);
            else
                break;
        }
    }
    file.close();
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        LUBISHEV_EXCEPTION("Cannot open file",
                           vaToStr("Could not open event file '%s'", qPrintable(fileName)));
    QTextStream out(&file);
    // Write data.
    out << start << "\n" << end << "\n";
    for (QStringList::const_iterator it = tags.begin(); it != tags.end(); ++it)
        out << *it << "\n";
}

QString DBDRecords::absPath(const QString &id) {
    return mDir + "/" + id;
}

QString DBDRecords::createID(time_t start) {
    QString yearMonth;
    QString day;
    dateToStr(start, yearMonth, day);
    return yearMonth + "/" + day + "_" + QUuid::createUuid().toString();
}

QStringList DBDRecords::dateFiles(time_t date) {
    QString yearMonth;
    QString day;
    dateToStr(date, yearMonth, day);
    QDir d(mDir + "/" + yearMonth);
    QStringList filters;
    filters << day + "_*";
    d.setNameFilters(filters);
    d.setFilter(QDir::Files);
    d.setSorting(QDir::Name);
    QStringList list = d.entryList();
    // Convert to IDs (paths).
    list.replaceInStrings(QRegExp("^"), yearMonth + "/");
    return list;
}

void DBDRecords::dateToStr(time_t date, QString &yearMonth, QString &day) {
    struct tm *timeinfo = localtime(&date);
    char buf[LUBISHEV_MAX_STRLEN];
    strftime(buf, LUBISHEV_MAX_STRLEN, "%Y%m", timeinfo);
    yearMonth = buf;
    strftime(buf, LUBISHEV_MAX_STRLEN, "%d", timeinfo);
    day = buf;
}

void DBDRecords::ensureDirs(const QString &id) {
    QDir d;
    QFileInfo fi(absPath(id));
    d.mkpath(fi.path());
}

} // namespace lubishev

